import unittest
import webtest
from pyramid import testing


class SomeClass(object):
    pass


class TestEncryptedCookieSessionFactory(unittest.TestCase):
    default_secret = b"seekrit!" * 4  # 32 bytes

    def _makeOne(self, **kw):
        from pyramid_nacl_session.session import EncryptedCookieSessionFactory

        kw.setdefault("secret", self.default_secret)
        return EncryptedCookieSessionFactory(**kw)

    def _makeOneFromSettings(self, **kw):
        from pyramid_nacl_session.utils import session_factory_from_settings

        kw.setdefault("session.secret", self.default_secret)
        return session_factory_from_settings(kw)

    def test_cookie(self):
        test_secure = self._build_cookie({"session.secure": "true"})
        test_secure_prefix = self._build_cookie({"pynacl.session.secure": "true"})
        test_not_secure = self._build_cookie({"session.secure": "false"})
        test_http_only = self._build_cookie({"session.httponly": "true"})
        test_http_secure = self._build_cookie(
            {"pynacl.session.secure": "true", "session.httponly": "true"}
        )
        test_cookie_name = self._build_cookie({"session.cookie_name": "_test_"})
        test_samesite = self._build_cookie({"session.samesite": "Strict"})

        _session_data = {
            "k1": "v1",
            "k2": 1,
            "k3": [1, 2, 3],
            "k4": {"t1": "v1", "t2": "v2"},
            "foo": ("some", "key"),
        }

        self.assertIn("secure;", test_secure)
        self.assertIn("secure;", test_secure_prefix)
        self.assertNotIn("secure;", test_not_secure)
        self.assertIn("HttpOnly;", test_http_only)
        self.assertIn("HttpOnly;", test_http_secure)
        self.assertIn("secure;", test_http_secure)
        self.assertIn("_test_=", test_cookie_name)
        self.assertIn("SameSite=Lax", test_cookie_name)
        self.assertIn("SameSite=Strict", test_samesite)

        with self.assertRaises(ValueError):
            self._build_cookie({"session.secret": "invalid"})

        with self.assertRaises(TypeError):
            _not_serializable = _session_data.copy()
            _not_serializable["k5"] = SomeClass
            self._build_cookie({"session.serializer": "json"}, _not_serializable)

        _pickle_serializable = _session_data.copy()
        _pickle_serializable["k5"] = SomeClass()
        _pickle_serializable["k6"] = SomeClass

        self.assertTrue(
            self._build_cookie({"session.serializer": "pickle"}, _pickle_serializable)
        )

    def _build_cookie(self, kw, session_data=None):
        config = testing.setUp()
        factory = self._makeOneFromSettings(**kw)
        config.set_session_factory(factory)

        if session_data is None:
            session_data = {}

        def test_view(context, request):
            request.session.update(session_data)
            return "test"

        config.add_view(test_view, name="test", renderer="string")
        app = config.make_wsgi_app()
        testapp = webtest.TestApp(app)
        resp = testapp.get("/test")
        return resp.headers.get("set-cookie")

    def test_include_me(self):
        from pyramid.config import Configurator
        from pyramid.registry import Registry
        from pyramid.interfaces import ISessionFactory
        from pyramid_nacl_session import includeme

        settings = {"session.secret": self.default_secret}
        reg = Registry()
        config = Configurator(reg)
        config.add_settings(settings)
        config.include(includeme)
        config.commit()
        factory = self._makeOne()
        self.assertIs(type(config.registry.getUtility(ISessionFactory)), type(factory))

    def test_it(self):
        config = testing.setUp()
        factory = self._makeOne()
        config.set_session_factory(factory)
        state_found = []

        def begin_view(context, request):
            state_found.append(request.session.copy())
            request.session["state"] = 1
            return "begin"

        def end_view(context, request):
            state_found.append(request.session.copy())
            return "end"

        config.add_view(begin_view, name="begin", renderer="string")
        config.add_view(end_view, name="end", renderer="string")
        app = config.make_wsgi_app()
        testapp = webtest.TestApp(app)

        testapp.get("/begin")
        testapp.get("/end")
        self.assertTrue("session" in testapp.cookies)
        self.assertEqual(state_found[0], {})
        self.assertEqual(state_found[1], {"state": 1})



from binascii import unhexlify

from pyramid.session import JSONSerializer
from pyramid.session import PickleSerializer

from pyramid.settings import asbool

from .session import EncryptedCookieSessionFactory


def session_factory_from_settings(settings):
    """
    Return a Pyramid session factory using PyNaCl session settings
    supplied from a Paste configuration file.
    """

    prefixes = ("session.", "pynacl.session.")
    options = {}
    bool_options = {"secure", "httponly", "set_on_exception"}
    serializers = {"json": JSONSerializer, "pickle": PickleSerializer}

    # Pull out any config args meant for PyNaCl session, if there are any.
    for k, v in settings.items():
        for prefix in prefixes:
            if k.startswith(prefix):
                option_name = k[len(prefix) :]

                if option_name in bool_options:
                    v = asbool(v)
                elif option_name == "serializer":
                    if v in serializers:
                        v = serializers[v]()
                elif option_name == "secret":
                    if not isinstance(v, bytes):
                        v = unhexlify(v)

                options[option_name] = v

    return EncryptedCookieSessionFactory(**options)



from .serializer import EncryptedSerializer
from .session import EncryptedCookieSessionFactory
from .scripts import generate_secret
from .utils import session_factory_from_settings

__all__ = [
    "EncryptedSerializer",
    "EncryptedCookieSessionFactory",
    "generate_secret",
    "session_factory_from_settings",
    "includeme",
]


def includeme(config):
    session_factory = session_factory_from_settings(config.registry.settings)
    config.set_session_factory(session_factory)
